// @TITLE "Shape.h: class Shape"
//
// Shape - an auxiliary class that is used to represent the shape of 
// an object, for the purpose of checking conformity between two objects.
// It is also used when constructing a new collection, to specify the 
// size of the collection.

// David Kotz 1994

// $Id: Shape.h,v 2.16 95/12/17 00:07:50 dfk release Locker: dfk $

#ifndef SHAPE_H
#define SHAPE_H

#include <stdlib.h>
#include <stdio.h>
#include "macros.h"

class Shape {
public:
    Shape(CONST char k, CONST int d1, CONST int d2 = 1)
      : kind(k), dim1(d1), dim2(d2) {
	  INVARIANT3(dim1 >= 1 && dim2 >= 1, "Illegal shape %s", name());
      };

    int number(void) CONST { return dim1 * dim2; } // how many elements?

    char *name(void) CONST {	// for printing error messages
	char *nameString = new char[20];
	sprintf(nameString, "%c[%d][%d]", kind, dim1, dim2);
	return nameString;
	// NOTE: this storage is not deleted, and thus name() is not suitable
	// except for printing fatal errors
    }

    // two shapes conform if they have identical kinds and dimensions
    boolean operator==(CONST Shape& S) CONST { 
	return (kind == S.kind && dim1 == S.dim1 && dim2 == S.dim2); 
    }

private:
    CONST char kind;		// kind of the shape, eg, 'V' for vector

public:
// Vector and Matrix need to look at dim1 and dim2, so I leave them public
    CONST int dim1, dim2;	// size of each dimension
};

#endif // SHAPE_H
